//     Copyright 2011 Reuben D'Netto
//
//     This file is part of SMWebUI.
//
//     SMWebUI is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     SMWebUI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with SMWebUI.  If not, see <http://www.gnu.org/licenses/>.


#include "myapplication.h"
#include <QStringList>
#include <QRegExp>
#include <QFile>
#include <QDir>
#include <QUrl>

MyApplication::MyApplication(int argc, char *argv[]) : QApplication(argc, argv){
	qDebug("SMWebUI v1.0");
	qDebug("Copyright Reuben D'Netto 2011");
	qDebug("MyApplication::MyApplication");

	//initialise
	HttpPort = 8080;
	SmplayerPort = 8000;
	SmplayerHost = "localhost";
	server = new QTcpServer(this);
	connection = new SmplayerConnection(this);

	connect(server, SIGNAL(newConnection()), this, SLOT(NewConnection()));
	LoadSmplayerSettings();

	//parse arguments
	QRegExp hPort(" \\-hp ([0-9]+)");
	QRegExp sHost(" \\-sh ([^ ]+)");
	QRegExp sPort(" \\-sp ([0-9]+)");
	QString args = arguments().join(" ");

	if(arguments().contains("--help")){
		qDebug("SMWebUI v1.0");
		qDebug("smwebui [-hp <http port>] [-sh <smplayer host>] [-sp <smplayer port>] [--help]");
		qDebug();
		qDebug("Runs a webUI on the specified port (default 8080) for the instance of SMPlayer running on specified host using the specified port.");
		qDebug("If you specify a host other than localhost, you must manually specify the port number.");
		server->deleteLater();
		server = 0;

	}else if(hPort.indexIn(args) != -1){
		HttpPort = hPort.cap(1).toInt();
		if(HttpPort == 0) qWarning("Invalid http port.");

	}else if(sPort.indexIn(args) != -1){
		SmplayerPort = sPort.cap(1).toInt();
		if(SmplayerPort == 0) qWarning("Invalid smplayer port.");

	}else if(sHost.indexIn(args) != -1){
		SmplayerHost = sHost.cap(1);
	} //end if
}

int MyApplication::exec(){
	if(! server) return 1;

	if(! connection->initialize(SmplayerHost, SmplayerPort)){
		qWarning("Could not connect to SMPlayer.");
		return 2;
	} //end if



	qDebug("MyApplication::exec: Listening to %s on port %i...", getAddresses().toUtf8().data(), HttpPort);
	server->listen(QHostAddress::Any, HttpPort);


	return QApplication::exec();
}

void MyApplication::NewConnection(){
	qDebug("MyApplication::NewConnection");

	QTcpSocket* socket = server->nextPendingConnection();
	if(! socket) return;
	socket->waitForReadyRead(-1);

	QString input = QString(socket->readLine()); //'GET /index.html HTTP/1.1'
	qDebug("MyApplication::NewConnection: Received: '%s'", input.trimmed().toUtf8().data());


	QStringList tokens = input.split(QRegExp("[ \r\n][ \r\n]*")); //splits input into words
	QTextStream os(socket);
	os.setAutoDetectUnicode(true);

	if (tokens[0] == "GET") {
		//preprocessing
		QString path = tokens[1];
		path.replace("..", ""); //prevent root access
		path += (path.endsWith('/') ? "index.html" : "");

		//use QUrl to separate components
		QUrl url("http://localhost" + path);
		path = QDir::cleanPath(QDir::currentPath() + "/www/" + url.path());
		QString ext;

		//get extension
		if(path.contains('.')){
			ext = QString(path);
			ext.remove(QRegExp("^.*\\."));
		}else{
			ext = "";
		} //end if

		if(QFile::exists(path)){
			QString mime = GetMimeType(ext);
			os << "HTTP/1.0 200 OK\r\n";

			if(mime != "")
				os << "Content-Type: " << mime << "; charset=\"utf-8\"\r\n\r\n";
			os.flush();

			QFile file(path);
			file.open(QIODevice::ReadOnly);
			socket->write(file.readAll());
			file.close();

		}else if(url.path() == "/smplayer"){
			//commands mapped to smplayer interface
			//uses a loop so multiple commands can be issued with a single request

			QPair<QString, QString> key;
			foreach(key, url.queryItems()){
				qDebug("MyApplication::NewConnection: Processing command: '%s=%s'", key.first.toUtf8().data(), key.second.toUtf8().data());

				if(key.first == "function"){
					if(key.second == "play")
						connection->play();
					else if(key.second == "pause")
						connection->pause();
					else if(key.second == "stop")
						connection->stop();
					else if(key.second == "previous")
						connection->previous();
					else if(key.second == "next")
						connection->next();
					else
						qDebug("MyApplication::NewConnection: Unknown function '%s'", key.second.toUtf8().data());

				}else if(key.first == "action"){
					qDebug("MyApplication::NewConnection: Processing action '%s'", key.second.toUtf8().data());
					connection->sendQuery("f " + key.second);

				}else if(key.first == "get"){
					QString value = connection->getValue(key.second);
					qDebug("MyApplication::NewConnection: get: '%s'='%s'", key.second.toUtf8().data(), value.toUtf8().data());
					os << value;

				}else if(key.first == "open"){
					qDebug("MyApplication::NewConnection: Opening '%s'", key.second.toUtf8().data());
					connection->openFile(key.second);

				}else if(key.first == "playItem"){
					qDebug("MyApplication::NewConnection: Playing item #%s", key.second.toUtf8().data());
					connection->playItem(key.second.toInt());

				}else if(key.first == "remove"){
					qDebug("MyApplication::NewConnection: Removing item #%s", key.second.toUtf8().data());
					connection->removeItem(key.second == "all" ? -1 : key.second.toInt());

				}else if(key.first == "moveUp"){
					qDebug("MyApplication::NewConnection: Moving up item #%s", key.second.toUtf8().data());
					connection->moveItem(key.second.toInt(), -1);

				}else if(key.first == "moveDown"){
					qDebug("MyApplication::NewConnection: Moving down item #%s", key.second.toUtf8().data());
					connection->moveItem(key.second.toInt(), +1);

				}else if(key.first == "seek"){
					qDebug("MyApplication::NewConnection: Seeking to position %s", key.second.toUtf8().data());
					connection->seek((int) key.second.toFloat());
					qDebug("seek complete");

				}else if(key.first == "status"){
					QString value = connection->status();
					qDebug("MyApplication::NewConnection: Retrieving status: %s", value.toUtf8().data());
					os << value;

				}else if(key.first == "clipinfo"){
					ClipInfo data = connection->clipInfo();
					QString output =
							+ "({ \n"
							+ QString("\t\"filename\":\"%1\", \n").arg(data.filename)
							+ QString("\t\"position\":%1, \n").arg((int) data.position)
							+ QString("\t\"duration\":%1, \n").arg((int) data.duration)
							+ QString("\t\"title\":\"%1\", \n").arg(data.title)
							+ QString("\t\"artist\":\"%1\", \n").arg(data.artist)
							+ QString("\t\"author\":\"%1\", \n").arg(data.author)
							+ QString("\t\"album\":\"%1\", \n").arg(data.album)
							+ QString("\t\"genre\":\"%1\", \n").arg(data.genre)
							+ QString("\t\"date\":\"%1\", \n").arg(data.date)
							+ QString("\t\"track\":\"%1\", \n").arg(data.track)
							+ QString("\t\"copyright\":\"%1\", \n").arg(data.copyright)
							+ QString("\t\"comment\":\"%1\", \n").arg(data.comment)
							+ QString("\t\"software\":\"%1\", \n").arg(data.software)
							+ "})";
					qDebug("MyApplication::NewConnection: Retrieving clip info:\n%s", output.toUtf8().data());
					os << output;

				}else if(key.first == "playlist"){
					QList<PlaylistItem> playlist = connection->playlist();
					PlaylistItem item;
					QString output = "([\n";

					foreach(item, playlist)
						output += QString("\t({\"name\":\"%1\", \"path\":\"%2\", \"length\":%3 }), \n")
						.arg(item.name, item.path, QString::number(item.length));

					output += "])";
					qDebug("MyApplication::NewConnection: Retrieving playlist:\n%s", output.toUtf8().data());
					os << output;

				}else if(key.first == "volume"){
					QRegExp numeric("[0-9]+");

					if(numeric.indexIn(key.second) > -1)
						connection->setVolume(key.second.toInt());
					else
						os << connection->getVolume();

				}else if(key.first == "open"){
					connection->openFile(key.second);

				}else if(key.first == "add"){
					connection->addFile(key.second);

				}else if(key.first == "legacyMode"){
					os << (connection->PartialSupportOnly() ? "true" : "false");

				}else{
					qDebug("MyApplication::NewConnection: Unknown command");
				} //end if

			} //end foreach

		}else{
			os << ErrorMsg(404, "Not Found");
			qDebug("MyApplication::NewConnection: 404 Not Found: %s", + path.toUtf8().data());
		} //end if

	}else{
		os << ErrorMsg(400, "Bad Request");
	} //end if


	os << "\r\n";
	os.flush();

	socket->flush();
	socket->close();
	socket->deleteLater();
}

