//     Copyright 2011 Reuben D'Netto
//
//     This file is part of SMWebUI.
//
//     SMWebUI is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     SMWebUI is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with SMWebUI.  If not, see <http://www.gnu.org/licenses/>.


#include "smplayerconnection.h"
#include <QRegExp>
#include <QStringList>
#include <QList>

SmplayerConnection::SmplayerConnection(QApplication *parent) : QObject(parent){
	app = parent;
	socket = new QTcpSocket();

	connect(socket, SIGNAL(error(QAbstractSocket::SocketError)),
			this, SLOT(OnError(QAbstractSocket::SocketError)));
	connect(socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
			this, SLOT(OnStateChanged(QAbstractSocket::SocketState)));
}

bool SmplayerConnection::initialize(QString host, int port){
	qDebug("MyApplication::exec: Connecting to SMPlayer on %s:%i...", host.toUtf8().data(), port);
	socket->connectToHost(host, port);

	if(! socket->waitForConnected())
		return false;

	CheckVersion();
	return true;
}

void SmplayerConnection::CheckVersion(){
	//runs immediately after connection is created
	//the regex mechanism should be used if this is merged with the mainline, as the alternative method is not as accurate

	//examples of input string:
	//'SMPlayer 0.6.9 (SVN r3447)'	#ubuntu package
	//'SMPlayer 0.6.9+SVN-r3609'	#SVN version

	//wait for version to be written to socket
	socket->waitForReadyRead(1000);

#ifdef SMPLAYERCONNECTION_CHECKVERSION
	QRegExp version_rx("SMPlayer ([0-9]+\\.[0-9]+\\.[0-9]+).*SVN.*r([0-9]+)");
	const int MinimumRelease = 3610;

	QString input(socket->readAll().simplified());
	qDebug("SmplayerConnection::CheckVersion: Parsing '%s'", input.toUtf8().data());

	if(version_rx.indexIn(input) == -1){
		qWarning("SmplayerConnection::CheckVersion: Invalid version string: %s", input.toUtf8().data());

	}else{
		//QString version = version_rx.cap(1);
		int release = version_rx.cap(2).toInt();
		_partialSupport = (release < MinimumRelease);

		qDebug("SmplayerConnection::CheckVersion: Connected to SMPlayer r%d", release);
	} //end if

#else

	socket->readAll(); //discard version string

	//set to true if it doesn't support the new commands
	_partialSupport = ! sendQuery("help").contains("view status");
#endif

	if(_partialSupport)
		qWarning("SmplayerConnection::CheckVersion: This version of SMPlayer is outdated - not all functions will be available.");
}

QString  SmplayerConnection::sendQuery(QString query){
	socket->write((query + "\r\n").toUtf8().data());
	socket->waitForReadyRead(1000);
	QString response = socket->readAll();

	if(response.isEmpty())
		qWarning("SmplayerConnection::sendQuery: Command failed: '%s'", query.toUtf8().data());

	if(response.contains("Unknown command")){
		qWarning("SmplayerConnection::sendQuery: Invalid command: '%s'", query.toUtf8().data());
		return "";
	}else{
		return response.trimmed();
	} //end if
}

QList<PlaylistItem> SmplayerConnection::playlist(){
	QList<PlaylistItem> output;
	if(PartialSupportOnly()) return output;

	QString rawData = sendQuery("view playlist").trimmed();
	QStringList data = rawData.split(QRegExp("[\r\n]+"));
	if(rawData == "") return output;

	foreach(QString line, data){
		QStringList fields = line.split('\t');
		PlaylistItem item;

		item.path = fields.at(0);
		item.name = fields.at(1);
		item.length = fields.at(2).toInt();

		output.append(item);
	} //end foreach

	return output;
}

ClipInfo SmplayerConnection::clipInfo(){
	ClipInfo result;
	if(PartialSupportOnly()) return result;

	QString rawData = sendQuery("view clip info").trimmed();
	QStringList data = rawData.split(QRegExp("[\r\n]+"));
	if(rawData == "") return result;

	data.replaceInStrings(QRegExp(".*\t"), "");

	result.filename =	data.at( 0);
	result.position =	data.at( 1).toFloat();
	result.duration =	data.at( 2).toFloat();
	result.title =		data.at( 3);
	result.artist =		data.at( 4);
	result.author =		data.at( 5);
	result.album =		data.at( 6);
	result.genre =		data.at( 7);
	result.date =		data.at( 8);
	result.track =		data.at( 9);
	result.copyright =	data.at(10);
	result.comment =	data.at(11);
	result.software =	data.at(12);

	return result;

}
